<!--Panel that has 1/4 page width-->
<div class="col-sm-3 col-xs-3 text-center">
    {% if danger %} 
  <div class="panel panel-danger">
    {% else %} 
  <div class="panel panel-primary">
    {% endif %} 
    <div class="panel-heading">{{title}}</div>
    <div class="panel-body">{{body}}</div>
  </div>
</div>
